/*
    This file is part of Wordle Solver.
    Copyright (C) 2022  Laurent Poirrier

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef aux_f
#include <stddef.h>
#include <stdint.h>

#define aux_f(fn)		aux2_ ## fn
#define aux_index		uint32_t
#define aux_val			uint64_t
#define aux_cmp(a, b)		((a) < (b))
#endif

#define AUX_W0			16

// ***************************************************************************
// 
// ***************************************************************************
static void aux_f(insert) (size_t n, aux_index *d, aux_val *v)
{
	for (size_t i = 1; i < n; i++) {
		aux_index p = d[i];
		aux_val pc = v[p];
		
		size_t j = i;
		while ((j) && (aux_cmp(pc, v[d[j - 1]]))) {
			d[j] = d[j - 1];
			j--;
		}
		
		d[j] = p;
	}
}

// ***************************************************************************
// 
// ***************************************************************************
static void aux_f(merge) (size_t w0, size_t w1, aux_index *a, aux_index *d,
	aux_val *v)
{
	aux_index *d0 = d;
	aux_index *d1 = d + w0;
	
	aux_index p0 = *d0;
	aux_index p1 = *d1;
	aux_val p0c = v[p0];
	aux_val p1c = v[p1];
	
	while (1) {
		if (aux_cmp(p1c, p0c)) {
			*(a++) = p1;
			
			if (!(--w1))
				break;

			p1 = *(++d1);
			p1c = v[p1];
		} else {
			*(a++) = p0;
			
			if (!(--w0))
				break;
			
			p0 = *(++d0);
			p0c = v[p0];
		}
	}
	
	if (w0) {
		*(a++) = p0;
		while (--w0)
			*(a++) = *(++d0);
	} else if (w1) {
		*(a++) = p1;
		while (--w1)
			*(a++) = *(++d1);
	}
}

// ***************************************************************************
// 
// ***************************************************************************
static int aux_f(sort_swapper) (size_t n, aux_index *d, aux_index *a,
	aux_val *v)
{
	size_t o;
	for (o = 0; o + AUX_W0 <= n; o += AUX_W0)
		aux_f(insert) (AUX_W0, d + o, v);
	
	if (o + 1 < n)
		aux_f(insert) (n - o, d + o, v);
	
	int odd = 0;
	
	for (size_t w = AUX_W0; w < n; w *= 2) {
		for (o = 0; o + (w * 2) <= n; o += (w * 2))
			aux_f(merge) (w, w, a + o, d + o, v);
		
		if (o + w < n) {
			aux_f(merge) (w, n - o - w, a + o, d + o, v);
		} else {
			for (; o < n; o++)
				a[o] = d[o];
		}
		
		aux_index *t = a;
		a = d;
		d = t;
		odd = !odd;
	}
	
	return odd;
}

// ***************************************************************************
// 
// ***************************************************************************
void aux_f(sort) (size_t n, aux_index **d, aux_index **a, aux_val *v)
{
	if (aux_f(sort_swapper) (n, *d, *a, v)) {
		aux_index *t = *d;
		*d = *a;
		*a = t;
	}
}


#undef aux_f
#undef aux_index
#undef aux_val
#undef aux_cmp
#undef AUX_W0
