/*
    This file is part of Wordle Solver.
    Copyright (C) 2022  Laurent Poirrier

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include "word.h"
#include "dict.h"
#include "builtin_dict.h"
#include "setcover.h"

// ****************************************************************************
// 
// ****************************************************************************
void setcover(void)
{
	dict d;
	
	dict_init(&d);
	dict_builtin_full(&d);
	
	printf("Minimize obj:\n");
	
	for (dict_index j = 0; j < d.count; j++) {
		printf(" + _%s", word_str(d.words[j]).s);
		if ((j % 9) == 8)
			printf("\n");
	}
	
	// _ills
	const char *letter = "bcdfghjklmnprstvwyz";
	// _ests
	//const char *letter = "bfghjklnprtvwyz";
	// _ests but removing k, l and r from larks
	//const char *letter = "bfghjnptvwyz";
	
	int toc = strlen(letter);
	
	printf("\nSubject To\n");
	
	printf("cover%d: ", toc - 1);
	for (int l = 0; l < toc; l++) {
		printf(" + _%c", letter[l]);
	}
	printf(" >= %d\n", toc - 1);

	for (int l = 0; l < toc; l++) {
		printf("one_%c:\n", letter[l]);
		uint8_t c = letter[l];
		dict_index k = 0;
		
		for (dict_index j = 0; j < d.count; j++) {
			if (!word_has(d.words[j], c))
				continue;
			
			printf(" + _%s", word_str(d.words[j]).s);
			if ((k++ % 9) == 8)
				printf("\n");
		}
		
		printf("\n - _%c >= 0\n", c);
	}
	
	printf("Binaries\n");
	
	for (int l = 0; l < toc; l++) {
		printf(" _%c", letter[l]);
	}
	printf("\n");

	for (dict_index j = 0; j < d.count; j++) {
		printf(" _%s", word_str(d.words[j]).s);
		if ((j % 12) == 11)
			printf("\n");
	}
	
	printf("\nEnd\n");
}

// ****************************************************************************
// 
// ****************************************************************************
